@@ -1,4 +1,6 @@
 import os
+import random
+import shutil
 import logging
 
 from typing import *
@@ -157,7 +159,6 @@ class EvaluateTransformer:
 SAVING_DELIM = "^_^"
 wrappers_dict_type = Dict[str, Wrapper]
 wrappers_type = Union[Wrapper, List[Wrapper], wrappers_dict_type]
-repeat_result_type = Tuple[EvaluateTransformer, Union[List[ModelPattern], Dict[str, List[ModelPattern]]]]
 
 
 def _to_saving_path(identifier: str,
@@ -190,6 +191,16 @@ def load_task(task: Task) -> Wrapper:
     return next(iter(load(saving_folder=task.saving_folder).values()))
 
 
+class RepeatResult(NamedTuple):
+    experiments: Experiments
+    transformer: EvaluateTransformer
+    patterns: Union[None, Dict[str, List[ModelPattern]]]
+
+    @property
+    def models(self) -> Dict[str, List[Wrapper]]:
+        return {key: [m.model for m in value] for key, value in self.patterns.items()}
+
+
 def repeat_with(x: data_type,
                 y: data_type = None,
                 x_cv: data_type = None,
@@ -200,9 +211,10 @@ def repeat_with(x: data_type,
                 num_jobs: int = 4,
                 num_repeat: int = 5,
                 temp_folder: str = "__tmp__",
-                return_tasks: bool = False,
+                return_patterns: bool = True,
                 use_tqdm: bool = True,
-                **kwargs) -> Union[repeat_result_type, Dict[str, List[Task]]]:
+                **kwargs) -> RepeatResult:
+
     if isinstance(models, str):
         models = [models]
     if identifiers is None:
@@ -213,47 +225,25 @@ def repeat_with(x: data_type,
     kwargs.setdefault("trigger_logging", False)
     kwargs["verbose_level"] = 0
 
-    tasks = patterns = None
-    if num_jobs <= 1:
-        kwargs.setdefault("use_tqdm", False)
-        if return_tasks:
-            tasks = {}
-            for i in range(num_repeat):
-                for model, identifier in zip(models, identifiers):
-                    task = Task(i, model, identifier, temp_folder)
-                    task.fit(make, save, x, y, x_cv, y_cv, **kwargs)
-                    tasks.setdefault(identifier, []).append(task)
-        else:
-            patterns = {}
-            for model, identifier in zip(models, identifiers):
-                init_method = lambda: make(model, **kwargs)
-                train_method = lambda m: m.fit(x, y, x_cv, y_cv)
-                pattern_kwargs = {"init_method": init_method, "train_method": train_method}
-                patterns[identifier] = ModelPattern.repeat(num_repeat, **pattern_kwargs)
-    else:
-        load_task_ = None if return_tasks else load_task
-        results = Experiments().run(
-            load_task_, x, y, x_cv, y_cv,
-            models=models, identifiers=identifiers,
-            num_repeat=num_repeat, num_jobs=num_jobs,
-            use_tqdm=use_tqdm, temp_folder=temp_folder, **kwargs
-        )
-        if return_tasks:
-            tasks = results
-        else:
-            patterns = {
-                model: [ModelPattern(init_method=lambda: wrapper) for wrapper in wrappers]
-                for model, wrappers in results.items()
-            }
-
-    if return_tasks:
-        return tasks
-
-    first_patterns = patterns[identifiers[0]]
-    tr_data = first_patterns[0].model.tr_data
-    if len(identifiers) == 1:
-        patterns = first_patterns
-    return EvaluateTransformer(tr_data), patterns
+    experiments = Experiments(temp_folder, overwrite=False)
+    experiments.run(
+        None, x, y, x_cv, y_cv,
+        models=models, identifiers=identifiers,
+        num_repeat=num_repeat, num_jobs=num_jobs,
+        use_tqdm=use_tqdm, temp_folder=temp_folder, **kwargs
+    )
+    patterns = None
+    if return_patterns:
+        tasks_dict = experiments.tasks
+        wrappers = {model: [load_task(task) for task in tasks] for model, tasks in tasks_dict.items()}
+        patterns = {
+            model: [ModelPattern(init_method=lambda: m) for m in wrappers]
+            for model, wrappers in wrappers.items()
+        }
+    transformer = None
+    if patterns is not None:
+        transformer = EvaluateTransformer(patterns[identifiers[0]][0].model.tr_data)
+    return RepeatResult(experiments, transformer, patterns)
 
 
 def ensemble(patterns: List[ModelPattern],
@@ -282,7 +272,13 @@ def tune_with(x: data_type,
               verbose_level: int = 2,
               **kwargs) -> HPOBase:
 
+    if os.path.isdir(temp_folder):
+        print(f"{LoggingMixin.warning_prefix}'{temp_folder}' already exists, it will be overwritten")
+        shutil.rmtree(temp_folder)
+
     if isinstance(x, str):
+        data_config = kwargs.get("data_config", {})
+        data_config["task_type"] = task_type
         read_config = kwargs.get("read_config", {})
         delim = read_config.get("delim", kwargs.get("delim"))
         if delim is not None:
@@ -294,7 +290,7 @@ def tune_with(x: data_type,
             )
         if y is not None:
             read_config["y"] = y
-        tr_data = TabularData(task_type=task_type)
+        tr_data = TabularData(**data_config)
         tr_data.read(x, **read_config)
         y = tr_data.processed.y
         if x_cv is not None:
@@ -322,11 +318,19 @@ def tune_with(x: data_type,
             num_repeat=num_repeat, num_jobs=num_jobs_,
             models=model, identifiers=hash_code(str(params_)),
             temp_folder=temp_folder, return_tasks=True, **base_params
-        )
+        ).experiments.tasks
 
     def _converter(created: List[Dict[str, List[Task]]]) -> List[pattern_type]:
+        patterns = []
         wrappers = list(map(load_task, next(iter(created[0].values()))))
-        return [ModelPattern(init_method=lambda: m) for m in wrappers]
+        for m in wrappers:
+            predict_method = (lambda m_: lambda x_: m_.predict(x_, contains_labels=True))(m)
+            predict_prob_method = (lambda m_: lambda x_: m_.predict_prob(x_, contains_labels=True))(m)
+            patterns.append(ModelPattern(
+                predict_method=predict_method,
+                predict_prob_method=predict_prob_method
+            ))
+        return patterns
 
     if params is None:
         params = {
@@ -385,6 +389,7 @@ def _to_wrappers(wrappers: wrappers_type) -> wrappers_dict_type:
 def estimate(x: data_type,
              y: data_type = None,
              *,
+             contains_labels: bool = False,
              wrappers: wrappers_type = None,
              wrapper_predict_config: Dict[str, Any] = None,
              metrics: Union[str, List[str]] = None,
@@ -408,7 +413,7 @@ def estimate(x: data_type,
             if y is not None:
                 y = to_2d(y)
             else:
-                x, y = wrapper.tr_data.read_file(x)
+                x, y = wrapper.tr_data.read_file(x, contains_labels=contains_labels)
                 y = wrapper.tr_data.transform(x, y).y
             if metrics is None:
                 metrics = [k for k, v in wrapper.pipeline.metrics.items() if v is not None]
@@ -809,6 +814,42 @@ class Benchmark(LoggingMixin):
 
 # others
 
+def split_file(file: str,
+               export_folder: str,
+               *,
+               has_header: bool = None,
+               split: Union[int, float] = 0.1) -> Tuple[str, str]:
+    os.makedirs(export_folder, exist_ok=True)
+    with open(file, "r") as f:
+        data = f.readlines()
+    ext = os.path.splitext(file)[1]
+    if has_header is None:
+        has_header = ext == ".csv"
+    header = None
+    if has_header:
+        header, data = data[0], data[1:]
+    split1 = os.path.join(export_folder, f"split1{ext}")
+    split2 = os.path.join(export_folder, f"split2{ext}")
+
+    num_data = len(data)
+    indices = list(range(num_data))
+    if split < 1. or split == 1. and isinstance(split, float):
+        split = int(num_data * split)
+    random.shuffle(indices)
+
+    def _split(file_, indices_):
+        with open(file_, "w") as f:
+            if header is not None:
+                f.write(header)
+            for idx in indices_:
+                f.write(data[idx])
+
+    _split(split1, indices[:split])
+    _split(split2, indices[split:])
+
+    return split1, split2
+
+
 def make_toy_model(model: str = "fcnn",
                    config: Dict[str, Any] = None,
                    *,
